@@ -27,6 +27,33 @@ def test_read_lines_splits_lines():
                for line in lines)
 
 
+def lines_from_file(tmpdir, contents):
+    f = tmpdir.join('f.py')
+    # be careful to write the bytes exactly to avoid newline munging
+    f.write_binary(contents)
+    return processor.FileProcessor(f.strpath, options_from()).lines
+
+
+def test_read_lines_universal_newlines(tmpdir):
+    r"""Verify that line endings are translated to \n."""
+    lines = lines_from_file(tmpdir, b'# coding: utf-8\r\nx = 1\r\n')
+    assert lines == ['# coding: utf-8\n', 'x = 1\n']
+
+
+def test_read_lines_incorrect_utf_16(tmpdir):
+    """Verify that a file which incorrectly claims it is utf16 is still read
+    as latin-1.
+    """
+    lines = lines_from_file(tmpdir, b'# coding: utf16\nx = 1\n')
+    assert lines == ['# coding: utf16\n', 'x = 1\n']
+
+
+def test_read_lines_unknown_encoding(tmpdir):
+    """Verify that an unknown encoding is still read as latin-1."""
+    lines = lines_from_file(tmpdir, b'# coding: fake-encoding\nx = 1\n')
+    assert lines == ['# coding: fake-encoding\n', 'x = 1\n']
+
+
 @pytest.mark.parametrize('first_line', [
     '\xEF\xBB\xBF"""Module docstring."""\n',
     u'\uFEFF"""Module docstring."""\n',
@@ -46,6 +73,7 @@ def test_strip_utf_bom(first_line):
     (['#!/usr/bin/python', '# flake8: noqa', 'a = 1'], True),
     (['# flake8: noqa', '#!/usr/bin/python', 'a = 1'], True),
     (['#!/usr/bin/python', 'a = 1', '# flake8: noqa'], True),
+    (['#!/usr/bin/python', 'a = 1  # flake8: noqa'], False),
 ])
 def test_should_ignore_file(lines, expected):
     """Verify that we ignore a file if told to."""
@@ -321,3 +349,8 @@ def test_log_token(token, log_string):
 def test_count_parentheses(current_count, token_text, expected):
     """Verify our arithmetic is correct."""
     assert processor.count_parentheses(current_count, token_text) == expected
+
+
+def test_nonexistent_file():
+    with pytest.raises(IOError):
+        processor.FileProcessor("foobar.py", options_from())
